/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is input buffer thread routines

#include "sbagenr.h"

/* Start off the thread that fills the buffer
*/
void inbuf_start(int(*rout)(int *, int), int len)
{
 if (0 != (len & (len - 1)))
  scr_error("inbuf_start() called with length not a power of two");

 ib_read = rout;
 ib_len = len;
 inbuf = ALLOC_ARR(ib_len, int);
 ib_rd = 0;
 ib_wr = 0;
 ib_eof = 0;
 if(!opt_Q)
  warn("Initialising %d-sample buffer for mix stream", ib_len / 2);

 // Preload 75% of the buffer -- or at least attempt to do so;
 // errors/eof/etc will be picked up in the inbuf_loop() routine
 ib_wr = ib_read(inbuf, (ib_len * 3) / 4);

 // Start the thread off
 ib_isrun = 1;			// thread enable

#ifdef UNIX_MISC
 {
  pthread_t thread;
  if(0 != pthread_create(&thread, NULL, (void *)&inbuf_loop, NULL))
   scr_error("Failed to start input buffering thread");
 }
#endif
#ifdef WIN_MISC
 {
  static unsigned mtid_dummy;

  if(0 == (ib_hev = CreateEvent(NULL, FALSE, FALSE, NULL)))
   scr_error("Failed to create control event for input buffering thread");
  InitializeCriticalSection(&ib_cs);
  if(0 == (ib_het = (HANDLE)_beginthreadex(NULL, 0,
	(LPTHREAD_START_ROUTINE)&inbuf_loop, 0, 0, &mtid_dummy)))
   scr_error("Failed to start input buffering thread");
 }
#endif
}

/* cleanup inbuf thread (w/o input file) (wrong for U*X)
*/
void inbuf_stop(void)
{
 ib_isrun = 0;			// set 'STOP' to thread
 
#ifdef UNIX_MISC
 // it's bad, but now we only can do:
 delay(500);
#endif

#ifdef WIN_MISC
 if(ib_hev)
 {
  SetEvent(ib_hev);
#if DEBUG_CLEANUP
  debug("-- Wait for the completion input thread...");
#endif
 }

 if(ib_het)
 {
  if(WAIT_TIMEOUT == WaitForSingleObject(ib_het, 500))
  {
   scr_fprintf(stderr, "Mix Reading tread need some wait...\n");
   WaitForSingleObject(ib_het, INFINITE);
  }
  CloseHandle(ib_het);
  ib_het = NULL;
#if DEBUG_CLEANUP
  debug("-- Input thread done.");
#endif

  MFree(inbuf);		// windoz only, in U*IX if free by common rules
  inbuf = NULL;

 // MP3/OGG deinitialize now for *win only* -- for reuse
 // if somebody need in on U*X, he/she must be sure, that
 // the mix thread is dead
#ifdef OGG_DECODE
  if(IBST_OGG == ib_stream_type)
  {
   ogg_term();
   ib_stream_type = IBST_RAW;	// no need to call mp3_term() anymore
  }
#endif
#ifdef MP3_DECODE
  if(IBST_MP3 == ib_stream_type)
  {
   mp3_term();
   ib_stream_type = IBST_RAW;	// no need to call mp3_term() anymore
  }
#endif

  if(ib_hev)
  {
   CloseHandle(ib_hev);
   ib_hev = NULL;
   // if ib_hev was created, ib_cs was created too
   DeleteCriticalSection(&ib_cs);
#if DEBUG_CLEANUP
   debug("-- Input thread win stuff deleted");
#endif
  }
 }
#endif
}

/* The common read threed routine
*/
int THREAD inbuf_loop(void *dummy)
{
 int now = -1;
 int waited = 0;	// Used to bail out if the main thread dies for some reason
 int a;

 while(ib_isrun)
 {
  int rv;
  int rd;
  int wr, lib_wr;
  int cnt;

#ifdef WIN_MISC
  EnterCriticalSection(&ib_cs);
  rd = ib_rd;
  wr = ib_wr;
  LeaveCriticalSection(&ib_cs);
#else
  rd = ib_rd;
  wr = ib_wr;
#endif
  cnt = (rd - 1 - wr) & (ib_len - 1);

  if(cnt > ib_len - wr)  cnt = ib_len - wr;
  if(cnt > ib_len / 8) cnt = ib_len / 8;

  // Choose to only work in ib_len/8 units, although this is not
  // 100% necessary
  if(cnt < ib_len / 8)
  {
   // Wait a little while for the buffer to empty (minimum 1ms)
   if(waited > 10000 + ib_cycle)
   {
    scr_fprintf(stderr, "Mix stream halted for more than 10 seconds; aborting\n");
    ib_eof = 1;
#ifdef WIN_MISC
    _endthreadex(0);
#endif
    return 0;
   }
#ifdef WIN_MISC
   delay_thr(a = 1 + ib_cycle / 4, ib_hev);
#else
   delay(a = 1 + ib_cycle / 4);
#endif
   waited += a;
   continue;
  }
  waited = 0;

  rv = ib_read(inbuf + wr, cnt);
  // debug("ib_read %d-%d (%d) -> %d", wr, wr + cnt - 1, cnt, rv);
#if 1
  // we slill cut the tails of the input file -- there is
  // need some work to fix it...
  if(rv != cnt)
  {
   ib_eof = 1;

#ifdef WIN_MISC
   _endthreadex(0);
#endif

   return 0;
  }
#else
  // an incomplete attempt to play mix to the end of file
  if(rv != cnt)
  {
   if(!rv)
   {
    ib_eof = 1;

#ifdef WIN_MISC
    _endthreadex(0);
#endif

    return 0;
   }
   else		// we get less, than cnt bytes
   {		// so, lets fill the rest by zeros
    // warn("-- Tail: %d (%d)", rv, cnt);
    memset(inbuf + wr + rv, (cnt - rv) * sizeof(int), 0);
    rv = cnt;
   }
  }
#endif

#ifdef WIN_MISC
  EnterCriticalSection(&ib_cs);
  lib_wr = ib_wr = (wr + rv) & (ib_len - 1);
  LeaveCriticalSection(&ib_cs);
#else
  lib_wr = ib_wr = (wr + rv) & (ib_len - 1);
#endif

  // Whenever we roll over, recalculate 'ib_cycle'
  if(lib_wr < wr)
  {
   int prev = now;

   now = calcNow();
   if(prev >= 0 && now > prev)
    ib_cycle = now - prev;
   //debug("Input buffer cycle duration is now %dms", ib_cycle);
  }
 }

#ifdef WIN_MISC
 _endthreadex(0);
#endif
 return 0;
}

/* Read a chunk of int data from the input buffer.  This will
 *	always return enough data unless we have hit the end of the
 *	file, in which case it returns a lower number or 0.  If not
 *	enough data has been read by the input thread, then this
 *	thread pauses until data is ready -- but this should hopefully
 *	never happen.
 */
// TODO::Need more robast synch. with working thread!!
int inbuf_read(int *dst, int dlen)
{
 int rv = 0;
 int waited = 0;	// As a precaution, bail out if other thread hangs for some reason
 int a;

 while(dlen > 0)
 {
  int rd;
  int wr;
  int avail;
  int toend;

#ifdef WIN_MISC
  EnterCriticalSection(&ib_cs);
  rd = ib_rd;
  wr = ib_wr;
  LeaveCriticalSection(&ib_cs);
#else
  rd = ib_rd;
  wr = ib_wr;
#endif
  avail = (wr - rd) & (ib_len - 1);
  toend = ib_len - rd;

  if(avail > toend) avail = toend;
  if(avail > dlen)  avail = dlen;

  if(avail == 0)
  {
   if(ib_eof) return rv;

   // Necessary to wait for incoming mix data.  This should
   // never happen in normal running, though, unless we are
   // outputting to a file
   if(waited > 10000) 
    scr_error("Mix stream problem; waited more than 10 seconds for data; aborting");
   //debug("Waiting for input thread (%d)", ib_eof);
   delay(a = ((ib_cycle / 4 > 100)? 100 : (1 + ib_cycle / 4)));
   waited += a;
   continue;
  }
  waited = 0;

  memcpy(dst, inbuf + rd, avail * sizeof(int));
  dst += avail;
  dlen -= avail;
  rv += avail;

#ifdef WIN_MISC
  EnterCriticalSection(&ib_cs);
  ib_rd = (rd + avail) & (ib_len - 1);
  LeaveCriticalSection(&ib_cs);
#else
  ib_rd = (rd + avail) & (ib_len - 1);
#endif
 }
 return rv;
}

/* The END...
*/

