/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// This is graphical SBaGen stdio replacement

#include "winsbagen.h"		// all the system and local inludes -- here

/* some globals
*/
volatile int isSbgRun;		// FALSE to stop SBG
volatile int needNotifyExit;	// !=0, if notification of SBG exit need

/*
 * The Working Thread API Fuctions
 * --- ------- ------ --- --------
 */
/* flush the current text buffer int fflush(FILE *fp)
*/
int scr_fflush(FILE *fp)
{
 // PostThreadMessage(idthrUI, XM_STDIO, (WPARAM)(SCR_FFUSH), (LPARAM)0);
 TQuePostMessage(XM_STDIO, (WPARAM)(SCR_FFUSH), (LPARAM)0);
 return 0;		// always success
}

/* put the single character int putc(int c, FILE *fp)
*/
int scr_putc(int c, FILE *fp)
{
 WPARAM wp = SCR_SINGLE;

 switch(c)
 {
  case '\r':
   wp |= SCR_CR;
   break;

  case '\n':
   wp |= SCR_LF;
   break;

  default:
   wp |= SCR_NOEOL;		// probably don't need
   break;
 }
 // PostThreadMessage(idthrUI, XM_STDIO, wp, (LPARAM)c);
 TQuePostMessage(XM_STDIO,  wp, (LPARAM)c);
 return c;		// always success
}

/* the int printf(char *fmt, ...)
*/
int scr_printf(char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 return scr_generic((WPARAM)0, fmt, ap);
}

/* the int fprintf(FILE *fp, char *fmt, ...)
*/
int scr_fprintf(FILE *fp, char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 return scr_generic((WPARAM)0, fmt, ap);
}

/* the int vfprintf(FILE *fp, char *fmt, va_list ap)
*/
int scr_vfprintf(FILE *fp, char *fmt, va_list ap)
{
 return scr_generic((WPARAM)0, fmt, ap);
}

/* print error message and terminate current SBaGen Thread void error(char *fmt, ...)
*/
void scr_error(char *fmt, ...)
{
 va_list ap;

 va_start(ap, fmt);
 scr_generic((WPARAM)SCR_ERROR, fmt, ap);
 exitSBaGen(1);
}

/* generic format and output screen message helper
*/
int scr_generic(WPARAM wp, char *fmt, va_list ap)
{
 int len, olen;
 char *obuf;

 // check the last char in the format -- we can lose it!
 len = strlen(fmt);
 switch(fmt[len - 1])
 {
  case '\r':
   wp |= SCR_CR;
   break;

  case '\n':
   wp |= SCR_LF;
   break;

  default:
   wp |= SCR_NOEOL;
   break;
 }
 // try to allocate the buffer wich enough to get the result message
 // (this is not good, but simple solution)
 obuf = malloc(len += SCR_FROMAT_TEXT_PLUS);
 if(NULL == obuf)	// oops...
  return -1;		// not a tragedy?
 // ok, we can try to format our text
 olen = _vsnprintf(obuf, len, fmt, ap);
 if(olen < 0)		// we lose the tail of text
 {
  wp &= (~SCR_NOEOL);
  wp |= SCR_LF;		// close a too long string
  obuf[olen = len - 1] = '\0';	// rest in peace
 }
 // drop out the last CR/LF
 if(olen)
 {
  char *p = obuf + (olen - 1);

  // this code looks bad, but seems reliable...
  // [anybody can rewrite it]
  if(*p == '\r' || *p == '\n')
  {
   // it may not present in format!!
   if(0 == (wp & SCR_LF))	// it may be already closed
   {
    wp &= (~SCR_NOEOL);
    wp |= (*p == '\r')? SCR_CR : SCR_LF;
   }
   *p = '\0';
   --olen;
  }
 }
 // the obuf must be free by the other thread
 // PostThreadMessage(idthrUI, XM_STDIO, wp, (LPARAM)obuf);
 TQuePostMessage(XM_STDIO, wp, (LPARAM)obuf);
 return olen;
}

/* notify the main thread, that the SBG exit
*/
void scr_exit_notify(int code)
{
 if(needNotifyExit)
 {
  // PostThreadMessage(idthrUI, XM_SBEXIT, (WPARAM)0, (LPARAM)code);
  TQuePostMessage(XM_SBEXIT, (WPARAM)0, (LPARAM)code);
 }
}

#ifdef UNICODE

/* convert ASCII (ANSI) string to UTF16
*/
LPWSTR asc2uni(char *src, LPWSTR dst)
{
 int len = strlen(src) + 1;

 // dst may be NULL only if all stopped
 if(NULL == dst)
  dst = cmdMalloc(len * sizeof(WCHAR));

 MultiByteToWideChar(CP_ACP, 0, src, -1, dst, len);
 return dst;
}

/* convert UTF16 string to ASCII (ANSI)
*/
char *uni2asc(LPWSTR src, char *dst)
{
 int len = wcslen(src) + 1;

 // dst may be NULL only if all stopped
 if(NULL == dst)
  dst = cmdMalloc(len);

 WideCharToMultiByte(CP_ACP, 0, src, -1, dst, len, NULL, NULL);
 return dst;
}

#endif

#ifdef UNDER_CE
// CE I/O stuff

/* low level write bytes to file
*/
int _write(int hf, void *buf, int cnt)
{
 DWORD wr;

 if(!WriteFile((HANDLE)hf, buf, (DWORD)cnt, &wr, NULL))
  return -1;
 return wr == (DWORD)cnt? cnt : -1;	// check disk overflow
}

#endif

/* The End...
*/

