/*
 *	SBaGen - Sequenced Binaural Beat Generator
 *
 *	(c) 1999-2005 Jim Peters <jim@uazu.net>.  All Rights Reserved.
 *	For latest version see http://sbagen.sf.net/ or
 *	http://uazu.net/sbagen/.  Released under the GNU GPL version 2.
 *      +---------------------+
 *	|Use at your own risk.|
 *	+---------------------+
 *	Code remastered by unknown corp.,Inc., generally for internal purpose
 *	by Ivan Karamazov under GNU GPL V2 terms.
 *
 *	" This program is free software; you can redistribute it and/or modify
 *	  it under the terms of the GNU General Public License as published by
 *	  the Free Software Foundation, version 2.
 *
 *	  This program is distributed in the hope that it will be useful,
 *	  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	  GNU General Public License for more details. "
 *
 *	See the file COPYING for details of this license.
 *
 *	- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 *
 *	Some code fragments in the Win32 audio handling are based on
 *	code from PLIB (c) 2001 by Steve Baker, originally released
 *	under the LGPL (slDSP.cxx and sl.h).  For the original source,
 *	see the PLIB project: http://plib.sf.net
 *
 *	The code for the Mac audio output was based on code from the
 *	FINK project's patches to ESounD, by Shawn Hsiao and Masanori
 *	Sekino.  See: http://fink.sf.net
 */

// the screen output stdio functions for the GUI
// and the other GUI-interact stuff

#ifndef _INC_GUI_STDIO
#define _INC_GUI_STDIO

#ifdef __cplusplus
extern "C" {
#endif

// horisontal tabulation size
#define	TAB_SIZE	(8)		/* standard */

// the messages to interchange between threads
#define	XM_STDIO	(WM_USER + 0x10) /* stdio emulation */
#define	XM_SBEXIT	(WM_USER + 0x11) /* SBG exit notification */

// the flags for wParam (can be combined, probably):
#define	SCR_FFUSH	(0x0001)	/* no chars -- flush buffer */
#define	SCR_SINGLE	(0x0002)	/* the single character */
#define	SCR_NOEOL	(0x0004)	/* the string w/o end */
#define	SCR_CR		(0x0008)	/* the \r ended string */
#define	SCR_LF		(0x0010)	/* the \n ended string */
#define	SCR_ERROR	(0x0020)	/* call of error() */

// the buffer 'tail' for formating string
// (this is not good, but simple solution)
#define	SCR_FROMAT_TEXT_PLUS	(512 * 2)

/* some globals
*/
extern volatile int isSbgRun;		// FALSE to stop SBG
extern volatile int needNotifyExit;	// !=0, if notification of SBG exit need

/*
 * The API Functions
 * --- --- ---------
 */
/* flush the current text buffer int fflush(FILE *fp)
*/
int scr_fflush(FILE *fp);
/* put the single character int putc(int c, FILE *fp)
*/
int scr_putc(int c, FILE *fp);
/* the int printf(char *fmt, ...)
*/
int scr_printf(char *fmt, ...);
/* the int fprintf(FILE *fp, char *fmt, ...)
*/
int scr_fprintf(FILE *fp, char *fmt, ...);
/* the int vfprintf(FILE *fp, char *fmt, va_list ap)
*/
int scr_vfprintf(FILE *fp, char *fmt, va_list ap);
/* print error message and terminate current SBaGen Thread void error(char *fmt, ...)
*/
void scr_error(char *fmt, ...);
/* generic format and output screen message helper
*/
int scr_generic(WPARAM wp, char *fmt, va_list ap);
/* notify the main thread, that the SBG exit
*/
void scr_exit_notify(int code);

#ifdef UNICODE
/* convert ASCII (ANSI) string to UTF16
*/
LPWSTR asc2uni(char *src, LPWSTR dst);
/* convert UTF16 string to ASCII (ANSI)
*/
char *uni2asc(LPWSTR src, char *dst);
#endif

/*
 * The API Macros
 * --- --- ------
 */
#define	CHECK_EXIT_CONDITIONS	do { \
	if(!isSbgRun) exitSBaGen(0); } while(0)

#ifdef UNDER_CE

/* low level write bytes to file
*/
int _write(int hf, void *buf, int cnt);

// Thread handle functions::
#define _beginthreadex(sec, stk, prg, prm, flg, id)	CreateThread( \
	(sec), (stk), (prg), (prm), (flg), (id))
#define	_endthreadex(code)	ExitThread(code)

// errno stuff:
#define	errno			GetLastError()

// rewind:
#define	rewind(fp)		fseek((fp), 0, SEEK_SET)

#endif

/*
 * gui_ctrl.c stuff
 * ---------- -----
 */
extern volatile int isChMute[];	// mute channel's flags


#ifdef __cplusplus
}
#endif

#endif	/* _INC_GUI_STDIO */

/* The END...
*/

